import { __awaiter } from 'tslib';
import { merge, of } from 'rxjs';
import { map, catchError, startWith, switchMap } from 'rxjs/operators';
import { DatePipe } from '@angular/common';
import { MomentDateAdapter } from '@angular/material-moment-adapter';
import { utc } from 'moment/moment';
import { Injectable, NgModule, defineInjectable, inject, EventEmitter, Component, ViewChild, Output, Input } from '@angular/core';
import { BrowserModule } from '@angular/platform-browser';
import { FormsModule } from '@angular/forms';
import { HttpClient, HttpHeaders, HttpClientModule } from '@angular/common/http';
import { BrowserAnimationsModule } from '@angular/platform-browser/animations';
import { MatTableDataSource, MatPaginator, MatSort, DateAdapter, MatTableModule, MatSortModule, MatInputModule, MatPaginatorModule, MatButtonModule, MatSelectModule, MatDatepickerModule, MatNativeDateModule, MatProgressSpinnerModule, MatListModule, MatFormFieldModule, MatIconModule, MatGridListModule } from '@angular/material';
import { MomentDatetimeAdapter, MatMomentDatetimeModule } from '@mat-datetimepicker/moment';
import { DatetimeAdapter, MAT_DATETIME_FORMATS, MatDatetimepickerFilterType, MatDatetimepickerModule } from '@mat-datetimepicker/core';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class MatDynamicTableService {
    /**
     * @param {?} http
     */
    constructor(http) {
        this.http = http;
    }
    /**
     * @param {?} valueColumns
     * @param {?} pageSize
     * @param {?} pageIndex
     * @param {?} api
     * @param {?} key
     * @param {?} direction
     * @return {?}
     */
    searchFilter(valueColumns, pageSize, pageIndex, api, key, direction) {
        /** @type {?} */
        const headers = new HttpHeaders({ 'Content-Type': 'application/json' });
        /** @type {?} */
        let valueColumnsParsed = JSON.parse(JSON.stringify(valueColumns));
        if (valueColumnsParsed.hasOwnProperty('details')) {
            delete valueColumnsParsed.details;
        }
        /** @type {?} */
        const requestBody = {
            columns: valueColumnsParsed,
            pagination: {
                pageSize,
                pageIndex,
            },
            sort: {
                key,
                direction
            }
        };
        return this.http.post(api, requestBody, { headers }).pipe(map((/**
         * @param {?} res
         * @return {?}
         */
        (res) => { return res; })));
    }
}
MatDynamicTableService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
/** @nocollapse */
MatDynamicTableService.ctorParameters = () => [
    { type: HttpClient }
];
/** @nocollapse */ MatDynamicTableService.ngInjectableDef = defineInjectable({ factory: function MatDynamicTableService_Factory() { return new MatDynamicTableService(inject(HttpClient)); }, token: MatDynamicTableService, providedIn: "root" });

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
const ɵ0 = {
    parse: {
        dateInput: 'L',
        monthInput: 'MMMM',
        yearInput: 'YYYY',
        timeInput: 'LT',
        datetimeInput: 'L LT'
    },
    display: {
        dateInput: 'L',
        monthInput: 'MMMM',
        yearInput: 'YYYY',
        datetimeInput: 'L LT',
        timeInput: 'LT',
        monthYearLabel: 'MMM YYYY',
        dateA11yLabel: 'LL',
        monthYearA11yLabel: 'MMMM YYYY',
        popupHeaderDateLabel: 'ddd, DD MMM'
    }
};
class MatDynamicTableComponent {
    /**
     * @param {?} backendApi
     */
    constructor(backendApi) {
        this.backendApi = backendApi;
        this.dataSource = new MatTableDataSource();
        this.displayedColumns = [];
        this.onSpecial = new EventEmitter();
        this.displayedTypeColumns = {};
        this.sortObject = {};
        this.resultsLength = 0;
        this.isLoadingResults = false;
        this.isRateLimitReached = false;
        this.dataStatus = 'loaded';
        this.typeColumns = {};
        this.columnsObject = {};
        this.valueColumns = {};
        this.datePipe = new DatePipe("en");
        this.today = utc().year(1929);
        this.tomorrow = utc().date(utc().date() + 1);
        this.min = this.today.clone().year(2018).month(10).date(3).hour(11).minute(10);
        this.max = this.min.clone().date(4).minute(45);
        this.start = this.today.clone().year(1930).month(9).date(28);
        this.filter = (/**
         * @param {?} date
         * @param {?} type
         * @return {?}
         */
        (date, type) => {
            switch (type) {
                case MatDatetimepickerFilterType.DATE:
                    return date.year() % 2 === 0 &&
                        date.month() % 2 === 0 &&
                        date.date() % 2 === 0;
                case MatDatetimepickerFilterType.HOUR:
                    return date.hour() % 2 === 0;
                case MatDatetimepickerFilterType.MINUTE:
                    return date.minute() % 2 === 0;
            }
        });
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.typeColumns = this.displayedTypeColumns;
        this.columnsObject = this.displayedTypeColumns;
        this.getHeaderKeys();
        if (Object.entries(this.sortObject).length === 0) {
            this.error = new Error('sortObject is required');
            this.dataStatus = 'error';
            throw this.error;
        }
        if (Object.entries(this.typeColumns).length === 0) {
            this.error = new Error('displayedTypeColumns is required');
            this.dataStatus = 'error';
            throw this.error;
        }
        if (!this.api) {
            this.error = new Error('api is required');
            this.dataStatus = 'error';
            throw this.error;
        }
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        setTimeout((/**
         * @return {?}
         */
        () => {
            this.searchFilter();
        }));
    }
    /**
     * @return {?}
     */
    searchFilter() {
        return __awaiter(this, void 0, void 0, function* () {
            this.dataSource.data = [];
            this.dataStatus = 'loading';
            // Boolean Columns parse
            Object.keys(this.valueColumns).forEach((/**
             * @param {?} x
             * @return {?}
             */
            (x) => {
                if (this.valueColumns[x] === 'true') {
                    this.valueColumns[x] = true;
                }
                else if (this.valueColumns[x] === 'false') {
                    this.valueColumns[x] = false;
                }
                if (this.valueColumns[x] === null) {
                    this.valueColumns[x] = '';
                }
            }));
            /** @type {?} */
            let key;
            /** @type {?} */
            let direction;
            //Default sort
            if (this.sort.active === undefined || this.sort.direction === '') {
                key = this.sortObject.key;
                direction = this.sortObject.direction;
            }
            else {
                key = this.sort.active;
                direction = this.sort.direction;
            }
            this.dataSource.sort = this.sort;
            this.sort.sortChange.subscribe((/**
             * @return {?}
             */
            () => this.paginator.pageIndex = 0));
            merge()
                .pipe(startWith({}), switchMap((/**
             * @return {?}
             */
            () => {
                return this.backendApi.searchFilter(this.valueColumns, this.paginator.pageSize, this.paginator.pageIndex, this.api, key, direction);
            })), map((/**
             * @param {?} data
             * @return {?}
             */
            data => {
                this.isLoadingResults = false;
                this.isRateLimitReached = false;
                this.resultsLength = data.totals.length;
                return data;
            })), catchError((/**
             * @return {?}
             */
            () => {
                this.isLoadingResults = false;
                this.isRateLimitReached = true;
                return of([]);
            }))).subscribe((/**
             * @param {?} data
             * @return {?}
             */
            (data) => {
                if (!data.data || !data.totals) {
                    this.error = new Error('data and totals are required');
                    this.dataStatus = 'error';
                    throw this.error;
                }
                this.dataSource.data = data.data;
                if (this.dataSource.data.length == 0) {
                    this.dataStatus = 'noData';
                }
                if (this.dataSource.data.length > 0) {
                    this.dataStatus = 'loaded';
                }
            }));
        });
    }
    /**
     * @param {?} header
     * @return {?}
     */
    formatHeader(header) {
        if (header.includes('.')) {
            /** @type {?} */
            let splited = header.split(".");
            /** @type {?} */
            var result = splited[1].replace(/([A-Z])/g, " $1");
            /** @type {?} */
            var finalResult = result.charAt(0).toUpperCase() + result.slice(1);
            return finalResult;
        }
        else {
            /** @type {?} */
            var result = header.replace(/([A-Z])/g, " $1");
            /** @type {?} */
            var finalResult = result.charAt(0).toUpperCase() + result.slice(1);
            return finalResult;
        }
    }
    /**
     * @return {?}
     */
    getHeaderKeys() {
        // Iterating and setting static types e.g. type "date/text/number/boolean" else delete
        Object.keys(this.typeColumns).forEach((/**
         * @param {?} key
         * @return {?}
         */
        (key) => {
            if (this.typeColumns[key].toLowerCase() == "string" || this.typeColumns[key].toLowerCase() == "array" || this.typeColumns[key].toLowerCase() == "text") {
                this.typeColumns[key] = "text";
            }
            else if (this.typeColumns[key].toLowerCase() == "long" || this.typeColumns[key].toLowerCase() == "int" || this.typeColumns[key].toLowerCase() == "double" || this.typeColumns[key].toLowerCase() == "number") {
                this.typeColumns[key] = "number";
            }
            else if (this.typeColumns[key].toLowerCase() == "date" || this.typeColumns[key].toLowerCase() == "boolean" || this.typeColumns[key].toLowerCase() == "special") {
                this.typeColumns[key] = this.typeColumns[key].toLowerCase();
            }
            else {
                console.log(`Unsuported type ${key}:${this.typeColumns[key]}, column has been deleted`);
                delete this.typeColumns[key];
            }
        }));
        this.displayedColumns = Object.keys(this.typeColumns);
        Object.keys(this.typeColumns).forEach((/**
         * @param {?} key
         * @return {?}
         */
        (key) => {
            this.valueColumns[key] = '';
        }));
    }
    /**
     * @param {?} date
     * @return {?}
     */
    formatDate(date) {
        if (date) {
            /** @type {?} */
            const newDate = this._formatDate(date);
            return newDate;
        }
        else {
            return 'No Date';
        }
    }
    /**
     * @param {?} date
     * @param {?=} format
     * @return {?}
     */
    _formatDate(date, format) {
        if (format) {
            return this.datePipe.transform(date, format);
        }
        else {
            return this.datePipe.transform(date, 'MMM d, y, HH:mm:ss');
        }
    }
    /**
     * @param {?} word
     * @param {?} element
     * @return {?}
     */
    spliceWord(word, element) {
        if (word != null && word.includes('.')) {
            /** @type {?} */
            let splited = word.split(".");
            /** @type {?} */
            let first = splited[0];
            /** @type {?} */
            let second = splited[1];
            /** @type {?} */
            let result;
            if (element[first] === null || element[first].length < 1) {
                result = 'No Data';
                return result;
            }
            else {
                result = element[first][0][second];
                return result;
            }
        }
        else {
            /** @type {?} */
            let result;
            if (element[word] == null) {
                result = 'No Data';
                return result;
            }
            else {
                result = element[word];
                return result;
            }
        }
    }
    /**
     * @param {?} element
     * @return {?}
     */
    onSpecialClick(element) {
        this.onSpecial.emit(element);
    }
}
MatDynamicTableComponent.decorators = [
    { type: Component, args: [{
                selector: 'mat-dynamic-table',
                template: "<div class=\"example-container mat-elevation-z8\">\n\n  <mat-table #table [dataSource]=\"dataSource\" matSort>\n\n    <ng-container *ngFor=\"let disCol of typeColumns | keyvalue; let i = index\" matColumnDef={{disCol.key}}>\n\n      <mat-header-cell *matHeaderCellDef> \n\n        <div *ngIf=\"disCol.key != 'details'\" style=\"width: 25%\"></div>    \n\n          <mat-form-field *ngIf=\"disCol.value=='date' && disCol.key!=='details'\" style=\"text-align:center;width: 75%\" (click)=\"$event.stopPropagation();\" title=\"{{formatHeader(disCol.key)}}\">\n            <mat-datetimepicker-toggle [for]=\"datetimePicker2\" matSuffix></mat-datetimepicker-toggle>\n            <mat-datetimepicker #datetimePicker2 type=\"datetime\" startView=\"month\" openOnFocus=\"true\" timeInterval=\"5\"></mat-datetimepicker>\n            <input matInput placeholder={{formatHeader(disCol.key)}} name=\"startTransactionTimeStamp\" #startTransactionTimeStamp (dateInput)=\"searchFilter()\"\n              [min]=\"today\" [max]=\"tomorrow\" [matDatetimepicker]=\"datetimePicker2\" autocomplete=\"false\" [(ngModel)]=valueColumns[disCol.key]>\n          </mat-form-field>\n          \n          <mat-form-field *ngIf=\"disCol.value=='boolean' && disCol.key!=='details'\" style=\"text-align:center;width: 75%;\"(click)=\"$event.stopPropagation();\" title=\"{{formatHeader(disCol.key)}}\">\n            <mat-select [(ngModel)]=\"valueColumns[disCol.key]\" placeholder=\"{{formatHeader(disCol.key)}}\" style=\"text-align: center;width:75%\"\n              (selectionChange)=\"searchFilter()\" name={{disCol.key}} #{{disCol.key}}>\n              <mat-option value=\"\">All</mat-option> \n              <mat-option value=\"true\">True</mat-option> \n              <mat-option value=\"false\">False</mat-option> \n            </mat-select>\n          </mat-form-field>\n\n          <mat-form-field *ngIf=\"disCol.value=='text' && disCol.key!=='details'\" style=\"text-align:center;width: 75%\" (click)=\"$event.stopPropagation();\" title=\"{{formatHeader(disCol.key)}}\">\n            <input matInput type=\"text\" class=\"form-control\" style=\"text-align:center;border: none !important\" placeholder={{formatHeader(disCol.key)}} (change)=\"searchFilter()\" name={{disCol.key}} #{{disCol.key}} [(ngModel)]=valueColumns[disCol.key] >\n          </mat-form-field>\n\n          <mat-form-field *ngIf=\"disCol.value=='number' && disCol.key!=='details'\" style=\"text-align:center;width: 75%\" (click)=\"$event.stopPropagation();\" title=\"{{formatHeader(disCol.key)}}\">\n              <input matInput type=\"number\" class=\"form-control\" style=\"text-align:center;border: none !important\" placeholder={{formatHeader(disCol.key)}} (change)=\"searchFilter()\" name={{disCol.key}} #{{disCol.key}} [(ngModel)]=valueColumns[disCol.key] >\n          </mat-form-field>\n        <h6 *ngIf=\"disCol.key=='details'\" class=\"headerH6\" title=\"{{formatHeader(disCol.key)}}\"> Details </h6>\n        <div *ngIf=\"disCol.key != 'details'\" style=\"width: 25%\" mat-sort-header={{disCol.key}} (click)=\"searchFilter()\"></div>\n      </mat-header-cell>\n\n      <mat-cell style=\"text-align:center\" *matCellDef=\"let element;let i = index\" [style.visibility]=\"dataStatus==='loaded'?'visible':'hidden'\">\n        <span *ngIf=\"dataStatus == 'loaded' && disCol.value=='date'\">{{formatDate(element[disCol.key])}}</span>\n        <span *ngIf=\"dataStatus == 'loaded' && disCol.key == 'details'\" (click)=\"onSpecialClick(element)\">        \n          <i class=\"material-icons\" *ngIf=\"!element[disCol.key]\">info</i>{{element[disCol.key] ? element[disCol.key] : '' }}\n        </span> \n        <span *ngIf=\"dataStatus == 'loaded' && disCol.value != 'date'\">{{element[disCol.key]}}</span>\n      </mat-cell>\n\n    </ng-container>\n\n    <mat-header-row *matHeaderRowDef=\"displayedColumns\"></mat-header-row>\n\n    <mat-row *matRowDef=\"let row; columns: displayedColumns;\"></mat-row>\n\n  </mat-table>\n\n  <div *ngIf=\"dataStatus == 'loading'\" >\n      <mat-grid-list cols=\"4\" rowHeight=\"100px\">\n        <mat-grid-tile  [colspan]=\"1\" [rowspan]=\"2\"></mat-grid-tile>\n        <mat-grid-tile  [colspan]=\"2\" [rowspan]=\"2\"><mat-spinner></mat-spinner></mat-grid-tile>\n        <mat-grid-tile  [colspan]=\"1\" [rowspan]=\"2\"></mat-grid-tile>\n      </mat-grid-list>\n  </div>\n\n  <div *ngIf=\"dataStatus == 'noData'\" >\n      <mat-grid-list cols=\"4\" rowHeight=\"100px\">\n        <mat-grid-tile  [colspan]=\"1\" [rowspan]=\"1\"></mat-grid-tile>\n        <mat-grid-tile  [colspan]=\"2\" [rowspan]=\"1\"><h3>No Data for the selected filters</h3></mat-grid-tile>\n        <mat-grid-tile  [colspan]=\"1\" [rowspan]=\"1\"></mat-grid-tile>\n      </mat-grid-list>\n  </div>\n\n  <div *ngIf=\"dataStatus == 'error'\" >\n      <mat-grid-list cols=\"4\" rowHeight=\"100px\">\n        <mat-grid-tile  [colspan]=\"1\" [rowspan]=\"1\"></mat-grid-tile>\n        <mat-grid-tile  [colspan]=\"2\" [rowspan]=\"1\"><h3>{{ error }}</h3></mat-grid-tile>\n        <mat-grid-tile  [colspan]=\"1\" [rowspan]=\"1\"></mat-grid-tile>\n      </mat-grid-list>\n  </div>\n\n  <mat-paginator #paginator [length]=\"resultsLength\" [pageSize]=\"10\" [pageSizeOptions]=\"[10,20,50,100]\" (page)=\"searchFilter()\" style=\"width: 100%;\"></mat-paginator>\n\n</div>",
                providers: [
                    {
                        provide: DateAdapter,
                        useClass: MomentDateAdapter
                    },
                    {
                        provide: DatetimeAdapter,
                        useClass: MomentDatetimeAdapter
                    },
                    {
                        provide: MAT_DATETIME_FORMATS,
                        useValue: ɵ0
                    }
                ],
                styles: ["table{width:100%}th.mat-sort-header-sorted{color:#000}.mat-form-field{font-size:14px;width:100%}mat-cell{display:flex!important;justify-content:center!important}mat-header-cell{display:flex;justify-content:center;place-content:center}"]
            }] }
];
/** @nocollapse */
MatDynamicTableComponent.ctorParameters = () => [
    { type: MatDynamicTableService }
];
MatDynamicTableComponent.propDecorators = {
    paginator: [{ type: ViewChild, args: [MatPaginator,] }],
    sort: [{ type: ViewChild, args: [MatSort,] }],
    onSpecial: [{ type: Output }],
    mode: [{ type: Input }],
    displayedTypeColumns: [{ type: Input }],
    api: [{ type: Input }],
    sortObject: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class MatDynamicTable {
}
MatDynamicTable.decorators = [
    { type: NgModule, args: [{
                declarations: [MatDynamicTableComponent],
                imports: [
                    BrowserModule,
                    BrowserAnimationsModule,
                    HttpClientModule,
                    MatButtonModule,
                    MatSortModule,
                    MatTableModule,
                    MatInputModule,
                    MatPaginatorModule,
                    MatSelectModule,
                    MatDatepickerModule,
                    MatNativeDateModule,
                    MatProgressSpinnerModule,
                    MatListModule,
                    MatDatetimepickerModule,
                    MatMomentDatetimeModule,
                    MatFormFieldModule,
                    MatIconModule,
                    MatGridListModule,
                    FormsModule
                ],
                exports: [MatDynamicTableComponent]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { MatDynamicTableService, MatDynamicTableComponent, MatDynamicTable };

//# sourceMappingURL=mat-dynamic-table.js.map